/**************************************************************************
IAR +STM8S_StdPeriph_Lib_V2.1
Chip type               : STM8S003K3
MCU Core Clock frequency: 16 MHz
Project:
sinewave converter with multichannel (3cnl) 200Hz DCDC stage
****************************************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include <math.h>


#define RHV_upr r1
#define RHV_lwr r2
//#define UHV_max (u16)(((u32)Vdd*(RHV_upr+RHV_lwr))/RHV_lwr) //=U_max*100
u16 UHV_max;
u8 ADCub310;
u8 cmp_val;

#define Rb_upr r3
#define Rb_lwr r4
//#define Ub_max (u16)(((u32)Vdd*(Rb_upr+Rb_lwr))/Rb_lwr) //=U_max*100
u16 Ub_max;

#define A       PD_ODR_ODR7
#define B       PD_ODR_ODR6
#define C       PD_ODR_ODR5
#define D       PD_ODR_ODR3
#define E       PD_ODR_ODR2
#define F       PD_ODR_ODR1
#define G       PB_ODR_ODR5
#define DP      PB_ODR_ODR4
#define N0      PA_ODR_ODR3
#define N1      PB_ODR_ODR7
#define N2      PB_ODR_ODR6

enum {prs, rls, hold} btn[1];

#define inv_off TIM1->EGR = TIM1_EGR_BG;
#define inv_on  TIM1->BKR|= TIM1_BKR_MOE;

#define pp1A PC_ODR_ODR7
#define pp1B PE_ODR_ODR5
#define pp2A PC_ODR_ODR6
#define pp2B PC_ODR_ODR5
#define pp3A PC_ODR_ODR4
#define pp3B PC_ODR_ODR3

bool CNL1, CNL2, CNL3;

enum {pp_softstart, pp_work} pp_state=pp_softstart;

volatile u16 tim[7];

volatile bool adc3_ready;
volatile bool adc2_ready;
volatile u16 adc3val;
volatile u16 adc2val;

//bool restart_bit=1;
u16 mst=0;

u8 dbuf[3]={0,0,0}, dpp=0;

#define cpu_freq 16000000/2
#define cnt_res 0xff
#define pwm_freq  cpu_freq/(2*cnt_res)
#define base_freq_50 50 //Hz
#define s_step pwm_freq/(base_freq_50*2)
#define pwm_top 0xff
static u8 s_table [s_step/2];


__no_init static u32 ee_r1   @ 0x4000+4*0;
__no_init static u32 ee_r2   @ 0x4000+4*1;
__no_init static u32 ee_r3   @ 0x4000+4*2;
__no_init static u32 ee_r4   @ 0x4000+4*3;
__no_init static u32 ee_Vdd  @ 0x4000+4*4;
__no_init static u32 ee_ktr  @ 0x4000+4*5;
__no_init static u32 ee_Ubm  @ 0x4000+4*6;
__no_init static u32 ee_wt   @ 0x4000+4*7;
u32 r1;
u16 r2,r3,r4,Vdd,ktr,Ubm;

void gpio_cfg()
{
  PA_ODR= 1<<1; //pullup for reset
  PB_ODR= 3;
  PC_ODR= 0xff-6;
  PD_ODR= 0;
  PE_ODR= 0xff;
  PF_ODR= 0;

  PA_CR1= 0xff-4;
  PB_CR1= 0xff-12;
  PC_CR1= 0xff;
  PD_CR1= 0xff-1;
  PE_CR1= 0xff;
  PF_CR1= 0xff;

  PA_DDR= 0xff-4;
  PB_DDR= 0xff-12;
  PC_DDR= 0xff;
  PD_DDR= 0xff-1;
  PE_DDR= 0xff;
  PF_DDR= 0;
}
void clk_cfg()
{
 CLK->CKDIVR = 0;
 CLK->PCKENR1 = CLK_PCKENR1_TIM1|CLK_PCKENR1_TIM4;
}
void t1_cfg()
{
  {
   TIM1->ARRH    = 0;
   TIM1->ARRL    = 0xff;
   TIM1->PSCRL   = 1;
   TIM1->CCR1H=TIM1->CCR2H=0;
   TIM1->CCR1L=TIM1->CCR2L=0xff;
   TIM1->CR1 = TIM1_COUNTERMODE_CENTERALIGNED1|TIM1_CR1_URS;
   TIM1->CR2 = TIM1_CR2_CCPC;
   TIM1->RCR = 1;
   TIM1->IER = TIM1_IER_UIE;
  }

  TIM1->CR2  |= TIM1_CR2_MMS;
  TIM1->CCR4L = 0xfd;
  TIM1->CCMR4 = TIM1_OCMODE_PWM1;

  TIM1->CCER1 = TIM1_CCER1_CC1E|TIM1_CCER1_CC1NE|TIM1_CCER1_CC1NP;
  TIM1->CCER1|= TIM1_CCER1_CC2E|TIM1_CCER1_CC2NE|TIM1_CCER1_CC2NP;

  TIM1->CCMR1 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
  TIM1->CCMR2 = 0x40|TIM1_CCMR_OCxPE;

  TIM1->EGR   = TIM1_EGR_COMG;

  TIM1->DTR  = 28*2; // dt delay (*62ns)
  TIM1->CR1 |= TIM1_CR1_CEN;
  TIM1->BKR |= TIM1_LOCKLEVEL_2;

}
void t4_cfg()
{
  TIM4->ARR   = 250-1;
  TIM4->PSCR  = 7;      //16MHz/250/2exp7 -> 500 Hz interrupt
  TIM4->IER   = 1;
  TIM4->CR1   = 1;
}
void adc_cfg()
{
  ADC1->CR1  = ADC1_PRESSEL_FCPU_D18;
  ADC1->CR2  = ADC1_ALIGN_LEFT|ADC1_CR2_EXTTRIG;
  ADC1->CSR  = 2;
  ADC1->TDRL = 1<<3|1<<2;

  ADC1->CR1 |= ADC1_CR1_ADON;
}
void iwdg_cfg()
{
IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
IWDG_SetPrescaler(IWDG_Prescaler_128);
IWDG_SetReload(0);    //2ms
IWDG_Enable();
IWDG_ReloadCounter();
}

void convert_data (u16 d)
{
 for (u8 k=0;k<3;k++)
  {
   dbuf[k]=d%10;
   d/=10;
  }
}
void display_data ()
{
  static u8 pc;

  (pc)?(pc--):(pc = 2);
  N0=N1=N2=0; DP=1;

  switch (dbuf[pc])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;

  case 10: A=0; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 11: A=1; B=0; C=1; D=1; E=1; F=1; G=1; break;
  case 12: A=1; B=1; C=0; D=1; E=1; F=1; G=1; break;
  case 13: A=1; B=1; C=1; D=0; E=1; F=1; G=1; break;
  case 14: A=1; B=1; C=1; D=1; E=0; F=1; G=1; break;
  case 15: A=1; B=1; C=1; D=1; E=1; F=0; G=1; break;

  case ' ': A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 'u': A=1; B=1; C=0; D=0; E=0; F=1; G=1; break;
  case 'c': A=1; B=1; C=1; D=0; E=0; F=1; G=0; break;
  case 'b': A=1; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 't': A=1; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'l': A=1; B=1; C=1; D=0; E=0; F=1; G=1; break;
  case 'o': A=1; B=1; C=0; D=0; E=0; F=1; G=0; break;
  case 'A': A=0; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'H': A=1; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'C': A=0; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'P': A=0; B=0; C=1; D=1; E=0; F=0; G=0; break;
  case 'E': A=0; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'U': A=1; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 'F': A=0; B=1; C=1; D=1; E=0; F=0; G=0; break;
  case 'L': A=1; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'r': A=1; B=1; C=1; D=1; E=0; F=1; G=0; break;
  case '_': A=1; B=1; C=1; D=0; E=1; F=1; G=1; break;
  case 'd': A=1; B=0; C=0; D=0; E=0; F=1; G=0; break;
  case 'n': A=1; B=1; C=0; D=1; E=0; F=1; G=0; break;
  case 'G': A=0; B=1; C=0; D=0; E=0; F=0; G=1; break;
  case 'O': A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  default:  A=1; B=1; C=1; D=1; E=1; F=1; G=0; break; //-
 }

   switch (pc)
 {
  case 0: N0=1; if(dpp == 0) {DP=0;} break;
  case 1: N1=1; if(dpp == 1) {DP=0;} break;
  case 2: if(dbuf[2]){N2=1;} if(dpp == 2) {DP=0;} break;
  default: DP=1; break;
 }
 }
void scan_keys ()
{
#define key0   PF_IDR_IDR4

#define lp  25
#define lh  30
#define lr  5
#define noc 1

static u8 count[noc*3];
volatile u8 key_st[noc];

key_st[0] = key0;

 for(u8 i=0;i<noc;i++)
 {
  if (!key_st[i]) {count[i*3]++;count[i*3+2]=lr;}
  else if((key_st[i])&&(!count[i*3+2])) {count[i*3]=count[i*3+1]=0;btn[i]=rls;}
  else count[i*3+2]--;

  if ((count[i*3]>lp)&&(btn[i]!=hold)) {btn[i]=prs; count[i*3]=0; count[i*3+1]++;}
  if (count[i*3+1]>lh) {btn[i]=hold; count[i*3+1]=0;}
 }
}

void devcon();
void beep(u8, u8);

INTERRUPT_HANDLER(TIM1_X3, 11) //15625 Hz
{
u16 tmp_table;

static u8 i=0;
static bool rck=0, dir;

/*ADC measurement section */

 static u16 adc3_cps=0, adc2_cps=0; //ADC3_Vbat, ADC2_VHV
 static u8 adc3_cnt=0, adc2_cnt=0;
 u8 adc_tmp;

 if(ADC1->CSR&1)
 {
  if(!adc3_ready)
  {
   adc3_cps+= ADC1->DRH;

   if(adc3_cnt<3){adc3_cnt++;}
   else
   {
      adc3val = adc3_cps>>2;
      adc3_cnt = adc3_cps = 0;
      adc3_ready = 1;
   }
  }
 }
 else
 {
  adc_tmp = ADC1->DRH;

  if(!adc2_ready)
  {
   adc2_cps+= adc_tmp;
   if(adc2_cnt!=0xff){adc2_cnt++;}
   else
   {
     adc2val = adc2_cps>>8;
     adc2_cnt = adc2_cps = 0;
     adc2_ready = 1;
   }
  }
 }
 if(!i)ADC1->CSR=3; else ADC1->CSR=2;

/*end of ADC measurement section */

//ultrasonic bridge control section

 if(mst<0xff/2);
 else if(adc_tmp>cmp_val) mst=(u16)ADCub310*0xff/adc_tmp;
 else 	{mst=0x1ff;}

 tmp_table=(u16)((u32)s_table [i]*mst>>8);
 if(tmp_table>0xff){tmp_table=0xff;}

  if(!i)
  {
   dir=1;i=1;

   if(rck)
   {
    rck=0;
    TIM1->CCMR1 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
    TIM1->CCMR2 = 0x40;
   }
   else
   {
    rck=1;
    TIM1->CCMR1 = 0x40;
    TIM1->CCMR2 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
   }

   if(mst<0xff/2)mst+=5;

   TIM1->EGR|=TIM1_EGR_COMG;
  }
  else if((dir)&&(i<s_step/2-1))
  {
    TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
    i++;
  }
  else
  {
    dir=0; i--;
    TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
  }

//end of ultrasonic bridge control section 

/*push_pull control section */
{
 if(CNL1)
 {
  const u8 tssv = (u8)(s_step/2);
  static u8 ssv;

  if(pp_state==pp_softstart)ssv=tssv/10;
  else if(pp_state==pp_work)ssv=tssv/2;

  {
   static u8 pp_i=0;

   if(pp_i<=(tssv-2*ssv)/2)         {pp1A=pp1B=1;    pp_i++;}
   else if(pp_i<(tssv-2*ssv)/2+ssv) {pp1A=1;pp1B=0;  pp_i++;}
   else if(pp_i<=tssv-ssv)          {pp1A=pp1B=1;    pp_i++;}
   else if(pp_i<tssv)               {pp1A=0;pp1B=1;  pp_i++; if(pp_i==tssv)pp_i=0;}
   else pp_i=0,pp1A=pp1B=1;
  }

  if(CNL2)pp2A=pp1A,pp2B=pp1B; else pp2A=pp2B=1;
  if(CNL3)pp3A=pp1A,pp3B=pp1B; else pp3A=pp3B=1;
 }
 else pp1A=pp1B=pp2A=pp2B=pp3A=pp3B=1;
}
/*end of push_pull control section */

 TIM1->SR1 = 0;
}

INTERRUPT_HANDLER(TIM4_OVF, 23)     //500 Hz
{

 if(tim[0])tim[0]--;
 if(tim[1])tim[1]--;
 if(tim[2])tim[2]--;
 if(tim[3])tim[3]--;
 if(tim[4])tim[4]--;
 if(tim[5])tim[5]--;
 if(tim[6])tim[6]--;

 scan_keys ();
 display_data();
 
 IWDG->KR = IWDG_KEY_REFRESH;
 TIM4->SR1 = 0;
}


void main()
{
 gpio_cfg();
 clk_cfg();
 adc_cfg();
 asm("sim");
 ITC_SetSoftwarePriority(ITC_IRQ_TIM1_OVF, ITC_PRIORITYLEVEL_3);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM4_OVF, ITC_PRIORITYLEVEL_1);
 t1_cfg();
 t4_cfg();

 for(u8 i=0;i<s_step/2;i++) s_table[i]=(u8)((float)pwm_top*sin(((3.1416/2)*i)/(s_step/2-1)));


 if(ee_wt!=0xaf)
 {
  FLASH_Unlock(FLASH_MEMTYPE_DATA);
  ee_wt    = 0xaf;
  ee_r1    = 1000000;
  ee_r2    = 11000;
  ee_r3    = 30000;
  ee_r4    = 10000;
  ee_Vdd   = 500;
  ee_ktr   = 2700;
  ee_Ubm   = 1050;

  FLASH_Lock(FLASH_MEMTYPE_DATA);
 }
 {
  r1  =      ee_r1;
  r2  = (u16)ee_r2;
  r3  = (u16)ee_r3;
  r4  = (u16)ee_r4;
  Vdd = (u16)ee_Vdd;
  ktr = (u16)ee_ktr;
  Ubm = (u16)ee_Ubm;
 }

 UHV_max  = (u16)(((u32)Vdd*(RHV_upr+RHV_lwr))/RHV_lwr); 
 ADCub310 = (u16)(((u32)31000*0xff)/UHV_max);
 cmp_val  = (u8)(20000UL*0xff/UHV_max);
 Ub_max   = (u16)(((u32)Vdd*(Rb_upr+Rb_lwr))/Rb_lwr);
 
 iwdg_cfg();
 asm("rim");

w:
 {

  devcon();

 }
 goto w;
}

void devcon()
{
 static enum  {start,work,LoHV,LoBat,EHBat,off,ocp} dev_st=start;
 static bool itr=0, btn_trg=0;

 if(btn[0]==rls){btn_trg=0;}
 else if(btn[0]==hold&&!btn_trg){btn_trg=1;dev_st=off;itr=0;}


 switch (dev_st)
 {
  case start:
  {
    static bool softstart_trg=0;
    if(!itr)
    {
     itr=1;
     softstart_trg=0;
     adc3_ready=adc2_ready=0;

     inv_off;
     CNL3=CNL2=CNL1=0;

     for(u8 i=0;i<7;)
     {
      if(!tim[0])
      {
       tim[0]=50;
       if(i<6){dbuf[2]=dbuf[1]=dbuf[0]=10+i; dpp=9;}
       else {dbuf[2]=dbuf[1]=dbuf[0]=10;}
       i++;
      }
     }
     adc3_ready=adc2_ready=0;
     beep(10, 4);
    }

    if(!tim[0])
    {
     dbuf[2]=11;
     if(adc3_ready)
     {
      dbuf[2]=12;
      if(adc3val<(u32)Ubm*0xff/Ub_max)dev_st=LoBat;
      else if(adc3val>(u32)1600*0xff/Ub_max)dev_st=EHBat;
      else
      {
       static u8 softstart_cnt;
       
       if(!softstart_trg)
       {
        softstart_trg=1;
        softstart_cnt=0;
        pp_state=pp_softstart;
        CNL1=1;
        tim[0]=1000;
        
        dbuf[2]=13;
       }
       else
       {
        if(adc2_ready)
        {
         if(adc2val>(u32)ktr*10*0xff*90/UHV_max/100){dev_st=work; softstart_trg=0;}
         else if(softstart_cnt<8) tim[0]=250,softstart_cnt++;
         else {dev_st=LoHV; softstart_trg=0;}
         
         if(dbuf[2]==14)dbuf[2]=15; else dbuf[2]=14;
         adc2_ready=0;
        }
       }
      }
      adc3_ready=0;
     }
    }

	 if(dev_st!=start){itr=0;}
	}
  break;

  case work:
  {
	 if(!itr)
    {
		itr=1;
    pp_state=pp_work;
    CNL3=CNL2=CNL1=1;
		dbuf[2]='r'; dbuf[1]='u'; dbuf[0]='n';
		mst=0;
		inv_on;
		tim[3]=tim[2]=tim[1]=tim[0]=500;
    tim[6]=0x3ff;
    adc3_ready=adc2_ready=0;
    }

	 if(!tim[0]) //      
	 {
	  tim[0]=500;
    convert_data((u32)adc3val*Ub_max/(0xff*10)); //display Vbat value
    //convert_data(adc2val); 
    dpp=1;
   }

   {
    if(adc3_ready&&adc2_ready) //   
    {
     static u16 t_tmp2=0,t_tmp3=0, tmp2=0, tmp3=0;

     if((u32)adc2val*UHV_max<(u32)adc3val*ktr*90/100*Ub_max/100&&adc2val<(u8)((u32)42000*0xff/UHV_max)) //  HV     420V
     {
      if(!CNL2)    //  2 
      {
       CNL2=CNL3=1;            //   
       if(tmp2<0xffff&&tim[4])tmp2++;//         ,    2
       tim[2]=tim[3]=0xffff;
       t_tmp3=t_tmp2=tmp2; //  
       tim[6]=0x3ff; //    
      }
      else if(!CNL3) //   3 
      {
       CNL3=1;      // 3_ 
       if(tmp3<0xffff&&tim[4])tmp3++;
       tim[3]=0xffff;
       t_tmp3=tmp3; //    
       tim[6]=0x3ff;
      }
     }
     else if((u32)adc2val*UHV_max>(u32)adc3val*ktr*95/100*Ub_max/100) //  HV     
     {
      if(CNL3&&!tim[3]&&!t_tmp3){CNL3=0;tim[3]=500;tim[4]=2000;} // 3_    
      if(CNL2&&!CNL3&&!tim[2]&&!tim[3]&&!t_tmp2){CNL2=0;tim[4]=2000;} //  2_

      if(!tim[5]) //    
      {
       tim[5]=0xffff;
       if(!CNL2&&!tmp3&&tmp2)tmp2--;
       if(!CNL3&&tmp3)tmp3--;
      }

      if(!tim[3]&&t_tmp3){t_tmp3--;tim[3]=0xffff;} // 
      if(!tim[2]&&t_tmp2&&!CNL3){t_tmp2--;tim[2]=0xffff;}
     }

     //LO&Hi HV protection part
     {
      if(adc2val>(u8)((u32)26000*0xff/UHV_max))tim[6]=0x3ff; //  DC>260V  
      else if(!tim[6]&&adc2val<(u8)((u32)26000*0xff/UHV_max)) dev_st=LoHV; //   HV<260VDC - 
      if(adc2val>(u8)((u32)42000*0xff/UHV_max))CNL3=CNL2=CNL1=0; //HV>420V -   DC_DC
      else if (adc2val<(u8)((u32)33000*0xff/UHV_max))CNL1=1;
     }
     //end of LO&Hi HV protection part

     //LO bat protection part
     if(!tim[1])
     {
      static bool lbt_trg=0;

      tim[1]=500;

      if(adc3val<(u8)((u32)Ubm*0xff/Ub_max)&&!lbt_trg) lbt_trg=1;
      else if(adc3val<(u8)((u32)Ubm*0xff/Ub_max)&&lbt_trg)
      {
       lbt_trg=0;
       dev_st=LoBat;
      }
      else lbt_trg=0;
     }
     //end of LO bat protection part
     adc3_ready=adc2_ready=0;
    }
   }

     
   //overcurrent protection part
   {
    static bool ocp_trg=0;
    if(GPIOD->IDR&1==0)
    {
     if(!ocp_trg)ocp_trg=1;
     else {CNL3=CNL2=CNL1=ocp_trg=0;dev_st=ocp;}
    }
    else  ocp_trg=0;      
   }
   //end of overcurrent protection part   

	if(dev_st!=work){itr=0;}
  }
  break;

 case LoBat:
  {
	 if(!itr)
    {
		itr=1;
		inv_off;
    CNL3=CNL2=CNL1=0;
    beep(10, 4);
		dbuf[2]='L'; dbuf[1]='o'; dbuf[0]='b'; dpp=1;
		tim[0]=5000;
    }

	 if(!tim[0])
   {
    if(adc3_ready)
    {
     adc3_ready=0;
     if(adc3val>(u8)((u32)Ubm*115*0xff/Ub_max/100)) dev_st=start;
     else tim[0]=2500;
    }
   }

	if(dev_st!=LoBat){itr=0;}
  }
  break;

 case EHBat:
  {
	 if(!itr)
    {
		itr=1;
		inv_off;
    CNL3=CNL2=CNL1=0;
    beep(10, 4);
		dbuf[2]='E'; dbuf[1]='H'; dbuf[0]='b'; dpp=1;
		tim[0]=5000;
    }

	 if(!tim[0])dev_st=start;

	 if(dev_st!=EHBat){itr=0;}
  }
  break;

 case off:
  {
	 if(!itr)
    {
		itr=1;
    CNL3=CNL2=CNL1=0;
		inv_off;
    beep(10, 4);
		dbuf[2]='_'; dbuf[1]='_'; dbuf[0]='_'; dpp=5;
    }

    if(btn[0]==prs&&!btn_trg){btn_trg=1;dev_st=start;}


	if(dev_st!=off){itr=0;}
  }
  break;

  case LoHV:
  {
	 if(!itr)
    {
		itr=1;
    CNL3=CNL2=CNL1=0;
		inv_off;
    beep(10, 4);
		dbuf[2]='L'; dbuf[1]='o'; dbuf[0]='H'; dpp=1;
		tim[0]=5000;
    }

	 if(!tim[0])dev_st=start;

	 if(dev_st!=LoHV){itr=0;}
  }
  break;  

 case ocp:
  {
	 if(!itr)
    {
		itr=1;
    beep(200, 4);
		dbuf[2]='O'; dbuf[1]='C'; dbuf[0]='P'; dpp=5;
    }

    if(btn[0]==prs&&!btn_trg){btn_trg=1;dev_st=start;}


	if(dev_st!=off){itr=0;}
  }
  break;
  
 default: break;
 }
}

void beep(u8 time, u8 freq) // 0xff = 2,55s
{
  BEEP->CSR = BEEP_CALIBRATION_DEFAULT;
 
  if(freq==1)      BEEP->CSR |= BEEP_FREQUENCY_1KHZ;
	else if(freq==2) BEEP->CSR |= BEEP_FREQUENCY_2KHZ;
	else             BEEP->CSR |= BEEP_FREQUENCY_4KHZ;
	BEEP->CSR |= BEEP_CSR_BEEPEN;
	tim[1]=(u16)time*5;
	while(tim[1]){;}
	BEEP->CSR = BEEP_CALIBRATION_DEFAULT;
}


